/************************************************************************
 *   MODULE  : BUILD_PR.C
 *
 *   PURPOSE : draws the profile display
 *
 *   AUTHOR  : Ron Hawkins & Craig Woodfolk, SAIC (original)
 *             Kevin Russo, SAIC (1, 3, 4)
 *             Melissa Mathews, SAIC (2)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   25 Sept 92    n/a
 *   1          12 Oct 92     make display more user friendly and
 *                              change away from using hardwired coords
 *   2          17 Jun 93     converted xvt 2.0 calls to xvt 3.0
 *   3          21 Jun 93     update to MUSE Beta Release 3
 *   4          30 Jul 93     modified to work on all screen resolutions
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

/************************************************************************
 *   FUNCTION     : construct_profile
 *
 *   PURPOSE      : builds the profile display
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : none
 *
 *   CALLED FROM  : do_win503 in module LOSW503.C
 *                  do_update in module LOSW503.C
 *
 ************************************************************************/

#ifdef PROTO
void construct_profile(void)
#else
void construct_profile()
#endif
{
    printf("contruct_profile\n");
#if 0
   short xnum_pix, ynum_pix; /* number of pixels for window height & width */
   short max_elev, min_elev; /* maximum and minimum elevation              */
   short delta_y;            /* delta of max and min elevation values      */
   short i, j;               /* elevation heights counter                  */
   float xscale, yscale;     /* # of pixels per y_unit of radial distance  */
   short check_pnt;          /* comparison point to determine color        */
   PNT   lpnt;               /* holder for graphics cursor position        */
   PNT   drawPnt;            /* pixel drawing position                     */
   PNT   *scrn_pts;          /* pointer to a set of screen coords          */
   RCT   rct, bndrct;        /* window rectangle, bounding rectangle       */
   CPEN  pen;                /* drawing pen                                */
   CBRUSH brush;             /* drawing brush                              */
   GHANDLE pnthandle;        /* handle to a set of screen coords           */
   short pHMEM elev_hgts;    /* radial heights array                       */
   char  pHMEM prof_colr;    /* profile color array                        */
   char buf[80];             /* buffer for text draw on screen             */
   int pixlen;               /* length of the character buffer in pixels   */
   int len;                  /* character buffer length in chars           */
   int pixhgt;               /* height of the character buffer in pixels   */
   int xpix_inc, ypix_inc;   /* pixel increments for plotting              */
   float yval_inc, xval_inc; /* x and y increments for plotting            */

   extern PARAM_CALC param;  /* info about the product                     */
   extern HANDLES handle;    /* holds locations of all memory used in calcs*/
   extern WIN_INFO wininfo;  /* window handle:  wininfo.lookupTable503[0]  */
   extern MAP_TYPE mapinfo;  /* non-muse map information                   */
   extern SETTINGS_TYPE settings;  /* user entered settings                */
   extern FONT_SIZE_TYPE fontsize; /* system-wide font information         */
   extern FONT /* XVT_FNTID xvt_R3FNT */  small_font;         /* system-wide font                     */
   extern PREFS_TYPE sys_prefs;    /* system display prefs                 */

   static COLOR colr[] = { COLOR_RED, COLOR_GREEN, COLOR_BLUE,
                           COLOR_CYAN, COLOR_MAGENTA, COLOR_YELLOW,
                           COLOR_BLACK, COLOR_WHITE };

   /* lock down profile memory buffers */
   elev_hgts = (short pHMEM) xvt_gmem_lock ( handle.hgts );
   prof_colr = (char  pHMEM) xvt_gmem_lock ( handle.prfl );
   if ((elev_hgts == (short pHMEM)NULL) || (prof_colr == (char pHMEM)NULL))
   {
     /*      xvt_dm_post_error("Unable to lock down profile memory buffers");*/
      return;
   }

   /* initialize elevation maximum and minimum */
   max_elev = (short)ZERO;
   min_elev = (short)SHRT_MAX;

   /* calculate elevation maximum and minimum */
   for (i = 0; i < param.nhts_in_radial; i++)
   {
      if (elev_hgts[i] == UNKNOWN_DTED)
         continue;

      if (elev_hgts[i] > max_elev)
         max_elev = elev_hgts[i];
      if (elev_hgts[i] < min_elev)
         min_elev = elev_hgts[i];
   }

   /* scale to have margins at graph top and bottom */
   max_elev += (settings.observer.height > settings.target.height) ?
               settings.observer.height : settings.target.height;

   max_elev *= 1.10;
   min_elev *= 0.90;

   if (max_elev < 15)
   {
      max_elev = 15;
      min_elev = 0;
   }
   else
   {
      max_elev += 5;
      while ((max_elev % 5) != ZERO)
         max_elev += 1;

      min_elev -= 5;
      if (min_elev < (short)ZERO)
         min_elev = 0;
      while ((min_elev % 5) != ZERO)
         min_elev -= 1;
   }

   /* calculate elevation delta */
   delta_y     = max_elev - min_elev;

   if (wininfo.lookupTable503[0] != (WINDOW)NULL)
   {
      xvt_vobj_get_client_rect(wininfo.lookupTable503[0], &rct);
      xvt_rect_set(&rct, rct.left+60, rct.top+15, rct.right-20, rct.bottom-55);
   }
   else
      return;

   /* x & y window pixel sizes are tied to actual window width and height */
   xnum_pix = rct.right - rct.left;
   ynum_pix = rct.bottom - rct.top;

   /* calculate pixels per unit radial distance */
   xscale = (float) xnum_pix / (float) param.nhts_in_radial;

   /* calculate pixels per meter of elevation */
   yscale = (float) ynum_pix / (float)delta_y;

   /* calculate the pixel increment in the x and y direction */
   xpix_inc = (int)((float)xnum_pix / 22.0);
   ypix_inc = (int)((float)ynum_pix / 14.0);

   /* draw a grid */
   pen.pat = PAT_SOLID;
   pen.width = 1;
   pen.style = P_SOLID;
#if XVT_OS_ISUNIX == TRUE
   pen.color = COLOR_GRAY;
#else
   pen.color = COLOR_GRAY62;
#endif
   brush.pat = PAT_SOLID;
   brush.color = COLOR_BLACK;
   xvt_dwin_set_cbrush(wininfo.lookupTable503[0],&brush);
   xvt_dwin_set_cpen  (wininfo.lookupTable503[0],&pen);

   rct.right = (short)rct.left + (short)((float)xpix_inc * 22);
   rct.bottom = (short)rct.top + (short)((float)ypix_inc * 14);
   xvt_dwin_draw_rect(wininfo.lookupTable503[0],&rct);


   /* vertical lines first */
   for (i = 1; i < 22; i++)
   {
      lpnt.h = rct.left + (short)(i * xpix_inc);
      lpnt.v = rct.top;
      xvt_dwin_draw_set_pos (wininfo.lookupTable503[0],lpnt);
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
      lpnt.v = rct.bottom;
#else
      lpnt.v = rct.bottom - 1;
#endif      
      xvt_dwin_draw_line (wininfo.lookupTable503[0],lpnt);
   }

   /* now the horizontal lines */
   for (i = 1; i < 14; i++)
   {
      lpnt.h = rct.left;
      lpnt.v = rct.top + (short)(i * ypix_inc);
      xvt_dwin_draw_set_pos (wininfo.lookupTable503[0],lpnt);
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
      lpnt.h = rct.right;
#else
      lpnt.h = rct.right - 1;
#endif      
      xvt_dwin_draw_line (wininfo.lookupTable503[0],lpnt);
   }

   /* display graph text */
   yval_inc = (float)(max_elev - min_elev) / 14.0;
   xvt_R3FNT_win_set_font(wininfo.lookupTable503[0],&small_font, FALSE);
		/* if the expression: (FALSE) is TRUE, set the XVT_FS_SCALE by: 
		   XVT_FONT_STYLE_MASK style = xvt_font_get_style((XVT_FNTID?!)*&small_font); 
		   xvt_font_set_style((XVT_FNTID?!)*&small_font, style | XVT_FS_SCALE ); 
		   then set font: 
		   xvt_dwin_set_font(wininfo.lookupTable5..., (XVT_FNTID?!)*&small_font);
		*/
   xvt_dwin_set_fore_color(wininfo.lookupTable503[0],COLOR_WHITE);
   xvt_dwin_set_back_color(wininfo.lookupTable503[0],COLOR_BLACK);

   for (i=0; i<=14; i += 2)
   {
      if (i == 14)
      {
         if (sys_prefs.dist_units == METERS)
            sprintf(buf, "%ld", (long)min_elev);
         else
            sprintf(buf, "%ld", (long)((GFLOAT)min_elev * FEET_PER_METER));
      }
      else
      {
         if (sys_prefs.dist_units == METERS)
            sprintf(buf, "%ld", (long)(max_elev - (yval_inc*i)));
         else
            sprintf(buf, "%ld", (long)((max_elev - (yval_inc*i))
                                 * FEET_PER_METER));
      }

      pixlen = xvt_dwin_get_text_width(wininfo.lookupTable503[0], buf, -1);

      drawPnt.h = (short)rct.left - (short)5 - (short)pixlen;

      if (i == 14)
         drawPnt.v = (short)rct.bottom+ 
                     (short)((float)fontsize.size / 2.0);
      else
         drawPnt.v = (short)rct.top + (short)((float)fontsize.size / 2.0) +
                     (short)(ypix_inc * i);

      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, buf, -1);
   }

   strcpy(buf, "ELEVATION");
   pixhgt = strlen(buf) * fontsize.size;
   len = strlen(buf);
   for (i=0; i<len; i++)
   {
      pixlen = xvt_dwin_get_text_width(wininfo.lookupTable503[0],buf, 1);

      drawPnt.h = (short)rct.left - (short)50 - (short)((float)pixlen / 2.0);
      drawPnt.v = (short)rct.top + (short)((float)(ynum_pix-pixhgt) / 2.0) +
                  (short)(fontsize.size * i);

      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, buf, 1);

      strcpy(&buf[0], &buf[1]);
   }

   xvt_dwin_set_fore_color(wininfo.lookupTable503[0],COLOR_YELLOW);

   drawPnt.h = (short)rct.left - (short)50 - (short)((float)pixlen / 2.0) +
               (short)xvt_dwin_get_text_width(wininfo.lookupTable503[0], "E", -1);
   drawPnt.v = (short)rct.top + (short)((float)(ynum_pix-pixhgt) / 2.0);

   xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, "*", -1);

   xvt_dwin_set_fore_color(wininfo.lookupTable503[0],COLOR_WHITE);

   xval_inc = (float)(mapinfo.prof_dist / 22.0);

   for (i=0; i<=22; i += 2)
   {
      if (i == 22)
      {
         if (sys_prefs.dist_units == METERS)
            sprintf(buf, "%ld", (long)mapinfo.prof_dist);
         else
            sprintf(buf, "%ld", (long)(mapinfo.prof_dist * FEET_PER_METER));
      }
      else
      {
         if (sys_prefs.dist_units == METERS)
            sprintf(buf, "%ld", (long)(xval_inc * i));
         else
            sprintf(buf, "%ld", (long)((GFLOAT)(xval_inc * i) *
                                 FEET_PER_METER));
      }

      pixlen = xvt_dwin_get_text_width(wininfo.lookupTable503[0],buf, -1);

      if (i == 22)
         drawPnt.h = (short)rct.right - (short)((float)pixlen / 2.0);
      else
         drawPnt.h = (short)rct.left + (short)(xpix_inc * i) -
                     (short)((float)pixlen / 2.0);
      drawPnt.v = (short)rct.bottom + (short)fontsize.size + (short)ONE;

      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, buf, -1);
   }

   strcpy(buf, "RANGE TO TARGET");

   pixlen = xvt_dwin_get_text_width(wininfo.lookupTable503[0],buf, -1);

   drawPnt.h = (short)rct.left + (short)((float)(xnum_pix-pixlen) / 2.0);
   drawPnt.v = (short)rct.bottom + (short)((float)fontsize.size * 2.5);

   xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, buf, -1);

   xvt_dwin_set_fore_color(wininfo.lookupTable503[0],COLOR_YELLOW);

   drawPnt.h = (short)rct.left + (short)((float)(xnum_pix-pixlen) / 2.0) +
               (short)pixlen;
   drawPnt.v = (short)rct.bottom + (short)((float)fontsize.size * 2.5);

   xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v, "*", -1);

   drawPnt.h = (short)rct.left - (short)50;
   drawPnt.v = (short)rct.bottom + (short)50;

   if (sys_prefs.dist_units == METERS)
      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v,
           "*All measurements in meters", -1);
   else
      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v,
           "*All measurements in feet", -1);

   xvt_dwin_set_fore_color(wininfo.lookupTable503[0],COLOR_WHITE);

   pen.width = 2;

   /* get an array of points for the cubic spline */
   pnthandle = (GHANDLE)xvt_gmem_alloc((param.nhts_in_radial+1) * sizeof(PNT));
   if (pnthandle == (GHANDLE)NULL)
      xvt_dm_post_fatal_exit("Unable to allocate memory in construct_profile()");

   scrn_pts = (PNT *)xvt_gmem_lock(pnthandle);
   if (scrn_pts == (PNT *)NULL)
      xvt_dm_post_fatal_exit("Unable to lock down memory in construct_profile()");

   for (i=0, j=0; i<param.nhts_in_radial; i++)
   {
      if (elev_hgts[i] == UNKNOWN_DTED)
         continue;

      if (i == 0)
         scrn_pts[1].h = rct.left + 2;
      else if (i == (param.nhts_in_radial - 1))
         scrn_pts[j+1].h = rct.right - 1;
      else
         scrn_pts[j+1].h = (short)((float)xscale * (float)i + (float)rct.left);

      scrn_pts[j+1].v = ynum_pix - (yscale * (elev_hgts[i] - min_elev)) + rct.top;

      if ((i > 0) && (abs(scrn_pts[j+1].h - scrn_pts[j].h) > (int)ZERO))
         j++;
   }

   /* calculate the spline */
   makespline (scrn_pts, (long)j);

   /* get a starting point */
   if (elev_hgts[0] != UNKNOWN_DTED)
   {
      lpnt.h = rct.left+2;
      lpnt.v = ynum_pix - (yscale * (elev_hgts[0] - min_elev)) + rct.top;
   }
   else
   {
      for (i=1; i<param.nhts_in_radial; i++)
      {
         if (elev_hgts[i] != UNKNOWN_DTED)
         {
            lpnt.h = rct.left+2;
            lpnt.v = ynum_pix - (yscale * (elev_hgts[i] - min_elev)) + rct.top;
            break;
         }
         if (i == (param.nhts_in_radial-1))
         {
            lpnt.h = rct.left+2;
            lpnt.v = rct.bottom;
            break;
         }
      }
   }
   xvt_dwin_draw_set_pos (wininfo.lookupTable503[0],lpnt);

   for (i=lpnt.h+1, j=0; i <= rct.right-1; i++)
   {
      check_pnt = (short)((float)xscale * (float)j + (float)rct.left);

      if (i >= check_pnt)
         j++;

      if (j == param.nhts_in_radial)
         j = param.nhts_in_radial - 1;

      switch(prof_colr[j])
      {
         case DEF_VISIBLE:
            pen.color = COLOR_GREEN;
            break;
         case PROB_VISIBLE:
            pen.color = COLOR_YELLOW;
            break;
         case PROB_HIDDEN:
            pen.color = COLOR_MAGENTA;
            break;
         case DEF_HIDDEN:
            pen.color = COLOR_RED;
            break;
         case BEYOND_ACUITY:
            pen.color = COLOR_CYAN;
            break;
         default:
            lpnt.h = i;
            xvt_dwin_draw_set_pos (wininfo.lookupTable503[0],lpnt);
            break;
      }

      if (prof_colr[j] != ZERO)
      {
         xvt_dwin_set_cpen (wininfo.lookupTable503[0],&pen);

         lpnt.h = i;
         if (i == lpnt.h + 1)
            lpnt.v = ynum_pix - (yscale * (elev_hgts[0] - min_elev)) + rct.top;
         else if (i == rct.right - 1)
            lpnt.v = ynum_pix - (yscale * (elev_hgts[param.nhts_in_radial-1] -
                     min_elev)) + rct.top;
         else
            lpnt.v = eval(scrn_pts, lpnt.h);
         if (lpnt.v > rct.bottom)
            lpnt.v = rct.bottom;
         xvt_dwin_draw_line (wininfo.lookupTable503[0],lpnt);
      }
   }

   /* unlock the point memory */
   if (xvt_gmem_unlock (pnthandle) == FALSE)
      xvt_dm_post_fatal_exit("Unable to unlock point memory");
   if (xvt_gmem_free (pnthandle) == FALSE)
      xvt_dm_post_fatal_exit("unable to free point memory");

   /* draw a direct line from the observer to the target */
   if ((elev_hgts[0] != UNKNOWN_DTED) &&
       (elev_hgts[param.nhts_in_radial-1] != UNKNOWN_DTED))
   {
      pen.color = COLOR_WHITE;
      pen.width = 1;
      pen.pat = PAT_SOLID;
      pen.style = P_SOLID;
      xvt_dwin_set_cpen(wininfo.lookupTable503[0],&pen);

      lpnt.h = rct.left+2;
      lpnt.v = ynum_pix - (yscale * ((elev_hgts[0]+settings.observer.height) -
                           min_elev)) + rct.top;

      xvt_dwin_draw_set_pos (wininfo.lookupTable503[0], lpnt);

      lpnt.h = rct.right;
      lpnt.v = ynum_pix - (yscale * ((elev_hgts[param.nhts_in_radial-1]+
               settings.target.height) - min_elev)) + rct.top;

      xvt_dwin_draw_line (wininfo.lookupTable503[0], lpnt);
   }


   /* indicate target and observer height on display */
   if (elev_hgts[0] != UNKNOWN_DTED)
   {
      lpnt.h = rct.left+2;
      lpnt.v = ynum_pix - (yscale * ((elev_hgts[0]+settings.observer.height) -
                           min_elev)) + rct.top;
      pen.color = colr[sys_prefs.obs_col];;
      pen.width = 1;
      pen.style = P_SOLID;
      pen.pat = PAT_SOLID;
      xvt_dwin_set_cpen(wininfo.lookupTable503[0],&pen);
      brush.color = COLOR_WHITE;
      brush.pat = PAT_SOLID;
      xvt_dwin_set_cbrush(wininfo.lookupTable503[0],&brush);
      xvt_rect_set(&bndrct, lpnt.h - 4, lpnt.v - 4, lpnt.h + 4, lpnt.v + 4);

      if (sys_prefs.obs_shape == S_CIRCLE)
         xvt_dwin_draw_oval(wininfo.lookupTable503[0],&bndrct);
      else
         xvt_dwin_draw_rect(wininfo.lookupTable503[0],&bndrct);

      brush.color = colr[sys_prefs.obs_col];
      xvt_dwin_set_cbrush(wininfo.lookupTable503[0],&brush);
      xvt_rect_set(&bndrct, lpnt.h - 2, lpnt.v - 2, lpnt.h + 2, lpnt.v + 2);

      if (sys_prefs.obs_shape == S_CIRCLE)
         xvt_dwin_draw_oval(wininfo.lookupTable503[0],&bndrct);
      else
         xvt_dwin_draw_rect(wininfo.lookupTable503[0],&bndrct);

      drawPnt.h = (short)bndrct.right + (short)4;
      drawPnt.v = (short)bndrct.top - (short)4;
      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v,
                    "OBSERVER", -1);
   }

   if (elev_hgts[param.nhts_in_radial-1] != UNKNOWN_DTED)
   {
      lpnt.h = rct.right;
      lpnt.v = ynum_pix - (yscale * ((elev_hgts[param.nhts_in_radial-1]+
               settings.target.height) - min_elev)) + rct.top;
      pen.color = colr[sys_prefs.tgt_col];
      pen.width = 1;
      pen.style = P_SOLID;
      pen.pat = PAT_SOLID;
      xvt_dwin_set_cpen(wininfo.lookupTable503[0],&pen);
      brush.color = COLOR_WHITE;
      brush.pat = PAT_SOLID;
      xvt_dwin_set_cbrush(wininfo.lookupTable503[0],&brush);
      xvt_rect_set(&bndrct, lpnt.h - 4, lpnt.v - 4, lpnt.h + 4, lpnt.v + 4);

      if (sys_prefs.tgt_shape == S_CIRCLE)
         xvt_dwin_draw_oval(wininfo.lookupTable503[0],&bndrct);
      else
         xvt_dwin_draw_rect(wininfo.lookupTable503[0],&bndrct);

      brush.color = colr[sys_prefs.tgt_col];
      xvt_dwin_set_cbrush(wininfo.lookupTable503[0],&brush);
      xvt_rect_set(&bndrct, lpnt.h - 2, lpnt.v - 2, lpnt.h + 2, lpnt.v + 2);

      if (sys_prefs.tgt_shape == S_CIRCLE)
         xvt_dwin_draw_oval(wininfo.lookupTable503[0],&bndrct);
      else
         xvt_dwin_draw_rect(wininfo.lookupTable503[0],&bndrct);

      drawPnt.h = (short)bndrct.left - (short)4 -
           (short)xvt_dwin_get_text_width(wininfo.lookupTable503[0], "TARGET", -1);
      drawPnt.v = (short)bndrct.top - (short)4;
      xvt_dwin_draw_text(wininfo.lookupTable503[0], drawPnt.h, drawPnt.v,
                    "TARGET", -1);
   }

   /* unlock profile memory */
   if (xvt_gmem_unlock(handle.prfl) == FALSE)
      xvt_dm_post_fatal_exit("Unable to unlock profile color buffer");
   if (xvt_gmem_unlock(handle.hgts) == FALSE)
      xvt_dm_post_fatal_exit("Unable to unlock profile heights buffer");
#endif
}
