/*
@PHIGS Functions
*/

#ifndef H_PHIGS_FUNC
#define H_PHIGS_FUNC

#ifndef H_PHIGS_DEF
#include "phigsdef.h"
#endif          /* H_PHIGS_DEF */

/***************************************************************
@    pcell_array()
****************************************************************
Phigs Cell Array
*/



#if XVT_CC_PROTO
MUSE_API pcell_array(Prect *rectangle, Ppat_rep colr_array);
#else
MUSE_API pcell_array();
#endif
/*
Description:
The image contained in colr_array is stretched to fit the
rectangle.
*/

/***************************************************************
@    pclose_phigs()
****************************************************************
Close Phigs Graphics
*/
#if XVT_CC_PROTO
MUSE_API pclose_phigs(void);
#else
MUSE_API pclose_phigs();
#endif
/*
Description:
*/

/***************************************************************
@    pclose_structure()
****************************************************************
Close Phigs Structure
*/
#if XVT_CC_PROTO
MUSE_API pclose_struct(void);
#else
MUSE_API pclose_struct();
#endif
/*
Description:
*/

/***************************************************************
@    pclose_ws()
****************************************************************
Close Phigs Workstation
*/
#if XVT_CC_PROTO
MUSE_API pclose_ws(Pint ws_id);
#else
MUSE_API pclose_ws();
#endif
/*
Description:
A window is the PHIGS workstation
*/

/***************************************************************
@    pdel_struct()
****************************************************************
Delete a PHIGS structure
*/
#if XVT_CC_PROTO
MUSE_API pdel_struct(Pint struct_id);
#else
MUSE_API pdel_struct();
#endif
/*
Description:
Delete a PHIGS structure
*/

/***************************************************************
@    pemergency_close_phigs()
****************************************************************
Emergency Close Phigs
*/
#if XVT_CC_PROTO
MUSE_API pemergency_close_phigs(void);
#else
MUSE_API pemergency_close_phigs();
#endif
/*
Description:

*/

/***************************************************************
@    pinq_bitmap()
****************************************************************
Inquire bitmap from operating system.
*/
#if XVT_CC_PROTO
MUSE_API pinq_bitmap(Pint ws_id , GHANDLE bitmap );
#else
MUSE_API pinq_bitmap();
#endif
/*
Description:
This function will return a bitmap of the screen
from the underlying graphics system.
*/

/***************************************************************
@    pinq_open_wss()
****************************************************************
Inquire open workstation
*/
#if XVT_CC_PROTO
MUSE_API pinq_open_wss(Pint length, Pint start, Pint *error,
		       Pint_list *ws_id_list, Pint *total_length);
#else
MUSE_API pinq_open_wss();
#endif
/*
Description:
This function will determine whether a workstation
is open or not.
*/

/***************************************************************
@    pinq_wss_trans()
****************************************************************
Returns current transformation information.
*/
#if XVT_CC_PROTO
MUSE_API pinq_wss_trans(Pint ws, Pint *error_ind, Pupd_st *upd_st,
		       Plimit *cur_win_lim, Plimit *cur_vp_lim,
		       Pvec *scale_vector, Pvec *trans_vector);
#else
MUSE_API pinq_wss_trans();
#endif
/*
Description:
Returns the current setup of the display
transformation.
*/

/***************************************************************
@    popen_phigs()
****************************************************************
Open Phigs
*/
#if XVT_CC_PROTO
MUSE_API popen_phigs(char *error_file, size_t memory);
#else
MUSE_API popen_phigs();
#endif
/*
Description:
Must be the first PHIGS call.
*/

/***************************************************************
@    popen_struct()
****************************************************************
Open Phigs Structure
*/
#if XVT_CC_PROTO
MUSE_API popen_struct(Pint struct_id);
#else
MUSE_API popen_struct();
#endif
/*
Description:
Open an existing structure or create a new one.
*/

/***************************************************************
@    popen_ws()
****************************************************************
Open Phigs Workstation
*/
#if XVT_CC_PROTO
MUSE_API popen_ws(Pint ws_id, Pconnid *conn_id, Pint ws_type);
#else
MUSE_API popen_ws();
#endif
/*
Description:
*/

/***************************************************************
@    ppolyline()
****************************************************************
Phigs polyline
*/
#if XVT_CC_PROTO
MUSE_API ppolyline(Ppoint_list *point_list);
#else
MUSE_API ppolyline();
#endif
/*
Description:
Point_list contains the points to be connected using the
current line style and color.
*/

/***************************************************************
@    ppolymarker()
****************************************************************
Phigs polymaker
*/
#if XVT_CC_PROTO
MUSE_API ppolymarker(Ppoint_list *point_list);
#else
MUSE_API ppolymarker();
#endif
/*
Description:
Point_list contains the points at which to place the markers.
*/

/***************************************************************
@    ppost_struct()
****************************************************************
Post Phigs Structure
*/
#if XVT_CC_PROTO
MUSE_API ppost_struct(Pint ws_id, Pint struct_id, Pfloat priority);
#else
MUSE_API ppost_struct();
#endif
/*
Description:
Sends the PHIGS structure to a workstation.
PHIGS structures are created by opening, drawing into , and
closing them.
*/

/***************************************************************
@    pput_pal()
****************************************************************
Post Phigs Structure
*/
#if XVT_CC_PROTO
MUSE_API pput_pal(Pint ws_id, PALETTE *palette);
#else
MUSE_API pput_pal();
#endif
/*
Description:
Assigns the palette argument to the currently open
phigs workstation.  (A window is a PHIGS workstation.)
*/

/***************************************************************
@    pscale()
****************************************************************
Phigs Scale
*/
#if XVT_CC_PROTO
MUSE_API pscale(Pvec scale_vector, Pint error_id, Pmatrix m);
#else
MUSE_API pscale();
#endif
/*
Description:
Used to zoom the image.
*/

/***************************************************************
@    pset_line_colr_ind()
****************************************************************
Phigs Line Colour Index
*/
#if XVT_CC_PROTO
MUSE_API pset_line_colr_ind (Pint index);
#else
MUSE_API pset_line_colr_ind();
#endif
/*
Description:
Sets the line colour to one of the defined types.
*/

/***************************************************************
@    pset_linetype()
****************************************************************
Phigs Line Type
*/
#if XVT_CC_PROTO
MUSE_API pset_linetype(Pint linetype);
#else
MUSE_API pset_linetype();
#endif
/*
Description:
Sets the line type to one of the defined types.
*/

/***************************************************************
@    pset_linewidth()
****************************************************************
Phigs Line Width
*/
#if XVT_CC_PROTO
MUSE_API pset_linewidth(Pfloat width);
#else
MUSE_API pset_linewidth();
#endif
/*
Description:
Sets polyline line width.
*/

/***************************************************************
@    pset_marker_colr_ind()
****************************************************************
Phigs Marker Colour Index
*/
#if XVT_CC_PROTO
MUSE_API pset_marker_colr_ind (Pint index);
#else
MUSE_API pset_marker_colr_ind();
#endif
/*
Description:
Sets the marker colour to one of the defined types.
*/

/***************************************************************
@    pset_marker_type()
****************************************************************
Phigs Set Marker Type
*/
#if XVT_CC_PROTO
MUSE_API pset_marker_type(Pint markertype);
#else
MUSE_API pset_marker_type();
#endif
/*
Description:
Sets the marker type to one of the defined types.
*/

/***************************************************************
@    pset_text_colr_ind()
****************************************************************
Phigs Text Colour Index
*/
#if XVT_CC_PROTO
MUSE_API pset_text_colr_ind (Pint index);
#else
MUSE_API pset_text_colr_ind();
#endif
/*
Description:
Sets the text colour to one of the defined types.
*/

/***************************************************************
@    pset_ws_vp()
****************************************************************
Phigs set workstation viewport
*/
#if XVT_CC_PROTO
MUSE_API pset_ws_vp (Pint ws_id, Plimit *viewport);
#else
MUSE_API pset_ws_vp();
#endif
/*
Description:
Changes the area of the drawing surface.
*/

/***************************************************************
@    ptext()
****************************************************************
Phigs Text
*/
#if XVT_CC_PROTO
MUSE_API ptext(Ppoint *text_pt, char *text);
#else
MUSE_API ptext();
#endif
/*
Description:
Plots text at the text_pt position using the current
color and style.
*/

/***************************************************************
@    ptranslate()
****************************************************************
Phigs Translate
*/
#if XVT_CC_PROTO
MUSE_API ptranslate(Pvec trans_vector, Pint error_id, Pmatrix m);
#else
MUSE_API ptranslate();
#endif
/*
Description:
Scrolls the display.
*/

/***************************************************************
@    pupd_ws()
****************************************************************
Update the PHIGS workstation
*/
#if XVT_CC_PROTO
MUSE_API pupd_ws(Pint ws, Pregen_flag regen_flag);
#else
MUSE_API pupd_ws();
#endif
/*
Description:

*/

/***************************************************************
@    ws_is_open()
****************************************************************
Test to see is workstation is open.
*/
#if XVT_CC_PROTO
MUSE_API ws_is_open(Pint ws);
#else
MUSE_API ws_is_open();
#endif
/*
Description:
Commonly used non-phigs function to test to see if
workstation is open.
*/


#endif /* #ifndef H_PHIGS_FUNC */
