 /*
 * UTIL.C - Utilities for MUSE 1.0
 */

#include "muse.h"
#include "rgbdefs.h"
#define BYTES_PER_IO 10240L

/*
 * JLL some values of BYTES_PER_IO do not seem to work out too good on the
 * PC, 5120 seems a good number.
 */

#if XVT_CC_PROTO
ERRSTATUS
big_fwrite(FILE * file, GHANDLE ghandle, unsigned long size,
	   NUM_TYPE num_type, DATA_TYPE data_type)
#else
ERRSTATUS
big_fwrite(file, ghandle, size, num_type, data_type)
    FILE           *file;
    GHANDLE         ghandle;
    unsigned long   size;
    NUM_TYPE        num_type;
    DATA_TYPE       data_type;
#endif

/*
 * Writes an arbitrarily large chunk of data from the the memory location
 * associated with the global heap handle.
 */
{
    ERRSTATUS       status = STAT_SUCCESS;
    BYTE           *io_block;
    BYTE pHMEM      data;
    long            i, bytes_left, index = 0;
    long            bytes_to_write;
    size_t          items_written;
    long            file_position = 0, right_position = 0;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    io_block = (BYTE *) muse_malloc(BYTES_PER_IO, "io_block");
    if (io_block == (BYTE *) NULL)
    {
	printf("unable to allocate the io_block");
	return STAT_MEM_ALLOC_ERR;
    }
    bytes_left = size;

    right_position = ftell(file);
    while (bytes_left > 0)
    {
	file_position = ftell(file);
	if (file_position != right_position)
	{
	    printf("file position: %ld, should be: %ld",
		      file_position, right_position);
	    return STAT_FILE_WRITE_ERR;
	}
	bytes_to_write = BYTES_PER_IO;
	if (bytes_left < BYTES_PER_IO)
	    bytes_to_write = bytes_left;

	data = (BYTE pHMEM) xvt_gmem_lock(ghandle);
	if (data == (BYTE pHMEM) NULL)
	{
	    printf("Unable to lock data in big_fwrite");
	    return STAT_MEM_ALLOC_ERR;
	}
	switch (data_type)
	{
	  case COLOR_RGB:
	    for (i = 0; i < bytes_to_write; i++)
		io_block[i] = data[index + i];
	    break;
	  case COLOR_INDIRECT:
	    for (i = 0; i < bytes_to_write; i++)
		io_block[i] = data[index + i];
	    break;
	  case DATA:
	    switch (num_type)
	    {
	      case SINT_16BIT:
	      case UINT_16BIT:
	      case DTED_16BIT:
		for (i = 0; i < bytes_to_write; i++)
		    if (big_endian)
		    {
			if ((i % 2) == 0)
			    io_block[i] = data[index + i + 1];
			else
			    io_block[i] = data[index + i - 1];
		    }
		    else
			io_block[i] = data[index + i];
		break;
	      default:
		xvt_dm_post_fatal_exit("NUM_TYPE not supported");
	    }
	    break;
	  default:
	    xvt_dm_post_fatal_exit("DATA_TYPE not supported");
	}
	xvt_gmem_unlock(ghandle);

	items_written = fwrite(io_block, bytes_to_write, 1, file);
	if (items_written != 1)
	    status = STAT_FILE_WRITE_ERR;
	bytes_left = bytes_left - bytes_to_write;
	index = index + bytes_to_write;
	right_position = right_position + BYTES_PER_IO;
	if (pacify("Saving the raster data.", (short)
		   ((float) (size - bytes_left) / (float) size * 100.0)))
	{
	    muse_free(io_block, "io_block");
	    return (STAT_ABORT);
	}
    }
    pacify("Ready", 0 );
    muse_free(io_block, "io_block");
    return (status);

}

#if XVT_CC_PROTO
ERRSTATUS
big_fread(FILE * file, GHANDLE ghandle, unsigned long size,
	  NUM_TYPE num_type, DATA_TYPE data_type)
#else
ERRSTATUS
big_fread(file, ghandle, size, num_type, data_type)
    FILE           *file;
    GHANDLE         ghandle;
    unsigned long   size;
    NUM_TYPE        num_type;
    DATA_TYPE       data_type;
#endif

/*
 * reads an arbitrairily large chunk of data from the the memory location
 * associated with the handle into the file.
 */
{

    ERRSTATUS       status = STAT_SUCCESS;
    BYTE           *io_block;
    BYTE pHMEM      data;
    long            i, bytes_left, index = 0;
    long            bytes_to_read;
    size_t          items_read;
    long            file_position = 0, right_position = 0;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    io_block = (BYTE *) muse_malloc(BYTES_PER_IO, "io_block");
    if (io_block == (BYTE *) NULL)
    {
	printf("unable to allocate the io_block");
	return STAT_MEM_ALLOC_ERR;
    }
    bytes_left = size;

    right_position = ftell(file);
    while (bytes_left > 0)
    {
	file_position = ftell(file);
	if (file_position != right_position)
	{
	    printf("file position: %ld, should be: %ld",
		      file_position, right_position);
	    return STAT_FILE_READ_ERR;
	}
	bytes_to_read = BYTES_PER_IO;
	if (bytes_left < BYTES_PER_IO)
	    bytes_to_read = bytes_left;
	items_read = fread(io_block, bytes_to_read, 1, file);
	if (items_read != 1)
	    status = STAT_FILE_READ_ERR;

	data = (BYTE pHMEM) xvt_gmem_lock(ghandle);
	if (data == (BYTE pHMEM) NULL)
	{
	    printf("Unable to lock data in big_fread");
	    return STAT_MEM_ALLOC_ERR;
	}
	switch (data_type)
	{
	  case COLOR_RGB:
	    for (i = 0; i < bytes_to_read; i++)
		data[index + i] = io_block[i];
	    break;
	  case COLOR_INDIRECT:
	    for (i = 0; i < bytes_to_read; i++)
		data[index + i] = io_block[i];
	    break;
	  case DATA:
	    switch (num_type)
	    {
	      case SINT_16BIT:
	      case UINT_16BIT:
	      case DTED_16BIT:
		for (i = 0; i < bytes_to_read; i++)
		    if (big_endian)
		    {
			if ((i % 2) == 0)
			    data[index + i + 1] = io_block[i];
			else
			    data[index + i - 1] = io_block[i];
		    }
		    else
			data[index + i] = io_block[i];
		break;
	      default:
		xvt_dm_post_fatal_exit("NUM_TYPE not supported");
	    }
	    break;
	  default:
	    xvt_dm_post_fatal_exit("DATA_TYPE not supported");
	}
	xvt_gmem_unlock(ghandle);

	bytes_left = bytes_left - bytes_to_read;
	index = index + bytes_to_read;
	right_position = right_position + BYTES_PER_IO;
	if (pacify("Reading the raster data.", (short)
		   ((float) (size - bytes_left) / (float) size * 100.0)))
	{
	    muse_free(io_block, "io_block");
	    return (STAT_ABORT);
	}
    }
    pacify("Ready", 0 );
    muse_free(io_block, "io_block");
    return (status);
}

#if XVT_CC_PROTO
MUSE_API muse_error(ERRSTATUS status)
#else
MUSE_API muse_error(status)
    ERRSTATUS       status;
#endif

/*
 * Will eventually be completed to handle MUSE errors, Cleanup and quit for
 * CRITICAL errors, Severly warn for severe erors, Calmly warn for mild
 * erors, and return.
 */
{
    printf("muse_error was called: %hd", status);
    return (MUSE_API)0;
}



#if XVT_CC_PROTO
ERRSTATUS
lut2pal(LUT * lut, COLOR_PREF color_pref, PALETTE * pal)
#else
ERRSTATUS
lut2pal(lut, color_pref, pal)
    LUT            *lut;
    PALETTE        *pal;
#endif

/*
 * Generates a palette from a lut if color_pref = FAVOR_TERAIN Maps the lut
 * to the existing palette if color_pref != FAVOR_TERRAIN
 */
{
    GFLOAT          hueinc, satinc, valinc;
    GFLOAT          hue, sat, val;
    int             ihue, ishade, middle;
    GFLOAT          r, g, b;
    ERRSTATUS       status;
    int             i;
    int             index = 0;
    RGB             rgb;
    USHORT          pal_color;

    if (lut == (LUT *) NULL)
	xvt_dm_post_fatal_exit("lut required in lut2pal");
    if (pal == (PALETTE *) NULL)
	xvt_dm_post_fatal_exit("palette required in lut2pal");

    for (i = 0; i < MAX_LUT; i++)
    {
	if (lut->lut_desc[i].num_hues == 0 ||
	    lut->lut_desc[i].num_shades == 0)
	    break;

	hue = lut->lut_desc[i].hue_range[0];
	hueinc = (lut->lut_desc[i].hue_range[1] - lut->lut_desc[i].hue_range[0])
		  / (lut->lut_desc[i].num_hues);
	middle = (long) ((lut->lut_desc[i].num_shades / 2.0));
	for (ihue = 0; ihue < lut->lut_desc[i].num_hues; ihue++, hue += hueinc)
	{
	    sat = lut->lut_desc[i].shade_range[0].sat;
	    satinc = (lut->lut_desc[i].shade_range[1].sat - lut->lut_desc[i].shade_range[0].sat)
		      / (lut->lut_desc[i].num_shades - middle);
	    val = lut->lut_desc[i].shade_range[0].val;
	    valinc = (lut->lut_desc[i].shade_range[1].val - lut->lut_desc[i].shade_range[0].val)
		      / (lut->lut_desc[i].num_shades - middle);
	    for (ishade = 0;
		 ishade < lut->lut_desc[i].num_shades;
		 ishade++, sat += satinc, val += valinc)
	    {
		if (index >= MAX_COLORS)
		{
		    printf("The palette was truncated.");
		    pal->num_colors = index;
		    return (-1);
		}
		if (ishade == middle)
		{
		    sat = lut->lut_desc[i].shade_range[1].sat;
		    satinc = (lut->lut_desc[i].shade_range[2].sat - lut->lut_desc[i].shade_range[1].sat)
			      / (lut->lut_desc[i].num_shades - middle);
		    val = lut->lut_desc[i].shade_range[1].val;
		    valinc = (lut->lut_desc[i].shade_range[2].val - lut->lut_desc[i].shade_range[1].val)
			      / (lut->lut_desc[i].num_shades - middle);

		}
		status = hsv2rgb(hue, sat, val, &r, &g, &b);
		if (status)
		    printf("Error in hsv2rgb");
		if (color_pref == FAVOR_TERRAIN)
		{
		    /* generate new palette */
		    pal->rgb[index].r = (unsigned) (r * 65535.0);
		    pal->rgb[index].g = (unsigned) (g * 65535.0);
		    pal->rgb[index].b = (unsigned) (b * 65535.0);
		    lut->map[index] = index;
		}
		else
		{

		    /*
		     * map closest color in existing palette
		     */
		    rgb.r = r * 65535.0;
		    rgb.g = g * 65535.0;
		    rgb.b = b * 65535.0;
		    status = pal_nearest_color(pal, rgb, &pal_color);
		    lut->map[index] = pal_color;
		    if (status)
			xvt_dm_post_fatal_exit("Error in pal_nearest_color.");
		}
		index++;
	    }			/* end for (ishade) */
	}			/* end for(ihue) */
    }				/* end for(i), lut_descriptor loop */
    return (STAT_SUCCESS);
}


/*
 * NAME: hsv2rgb()
 * 
 * PURPOSE: Convert Hue, Saturation, Value color inputs to RGB values.
 * 
 * AUTHOR: Chris Hood    (Sverdrup Technology)
 * 
 * SUMMARY: #include "phigs.h" hsv2rgb(h, s, v, r, g, b, err); double h, s, v;
 * double *r, *g, *b; int *err;
 * 
 * DESCRIPTION: Given h in [0, 360] or undefined. s and v in [0, 1] Desired r,
 * g, b each in [0, 1]
 * 
 * RETURN VALUE: double r, g, b
 * 
 * SYSTEM DEPENDENT FUNCTIONS: None
 */


#if XVT_CC_PROTO
ERRSTATUS
hsv2rgb(GFLOAT h, GFLOAT s, GFLOAT v,
	GFLOAT * r, GFLOAT * g, GFLOAT * b)
#else
ERRSTATUS
hsv2rgb(h, s, v, r, g, b)
    GFLOAT          h, s, v, *r, *g, *b;
#endif
{
    int             i = 0;
    double          f, p, q, t;

    h = fmod(h, 360.0);
    if (s == 0)
    {
	*r = *g = *b = v;
	return (0);
    }
    else
    {
	h = h / 60;
	i = h;
	f = h - i;
	p = v * (1 - s);
	q = v * (1 - (s * f));
	t = v * (1 - (s * (1 - f)));
	switch (i)
	{
	  case 0:

	    *r = v;
	    *g = t;
	    *b = p;
	    break;

	  case 1:

	    *r = q;
	    *g = v;
	    *b = p;
	    break;

	  case 2:

	    *r = p;
	    *g = v;
	    *b = t;
	    break;

	  case 3:

	    *r = p;
	    *g = q;
	    *b = v;
	    break;

	  case 4:

	    *r = t;
	    *g = p;
	    *b = v;
	    break;

	  case 5:

	    *r = v;
	    *g = p;
	    *b = q;
	    break;
	}			/* End Switch */
    }				/* End Else */
    return (0);
}


#include <string.h>

#if 0 /*XVT_OS == XVT_OS_WIN*/
#define FILE_SEP_1 '\\'
#define FILE_SEP_2 '.'
#endif

#if 0 /*XVT_OS == XVT_OS_MAC*/
#define FILE_SEP_1 ':'
#define FILE_SEP_2 '.'
#endif

#if XVT_OS_ISUNIX
#define FILE_SEP_1 '/'
#define FILE_SEP_2 '.'
#endif

#if 0
#if XVT_CC_PROTO
MUSE_API file_spec_to_string(FILE_SPEC * file_spec, char *string)
#else
MUSE_API file_spec_to_string(file_spec, string)
    FILE_SPEC      *file_spec;
    char           *string;
#endif
{
    int             sz_path = SZ_FNAME;
    char            dir_string[SZ_FNAME];
    
    if (!xvt_fsys_convert_dir_to_str(&file_spec->dir, dir_string, sz_path))
    {
/*	printf("Unable to convert the directory into a string.");
	return (-1);*/
    }

#if XVTWS == MACWS
    sprintf(string, "%s%s",
	    dir_string, file_spec->name);
#else
    if ( dir_string[0] != '\0' )
       sprintf(string, "%s%c%s",
	    dir_string, FILE_SEP_1, file_spec->name);
    else
       sprintf(string, "%s", file_spec->name);
       
#endif

    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
MUSE_API string_to_file_spec(FILE_SPEC * file_spec, char *string)
#else
MUSE_API string_to_file_spec(file_spec, string)
FILE_SPEC      *file_spec;
char           *string;
#endif

/* scan backwards looking for FILE_SEP_1 and then FILE_SEP 2 */

{
    short           i = 0;
    size_t          count;
    char            dir_str[SZ_FNAME];

    /* search back from the end to find the type seperator and type */
    count = 0;
    i = strlen(string);
    while (i > 0 && string[i] != FILE_SEP_2)
    {
	i--;
	count++;
    }
    strncpy((char *) &file_spec->type, &string[i + 1], count);
    file_spec->type[count] = 0;


    /*
     * search back from the end to find the name seperator (name includes
     * type)
     */
    count = 0;
    i = strlen(string);
    while (i > 0 && string[i] != FILE_SEP_1)
    {
	i--;
	count++;
    }
    strncpy((char *) &file_spec->name, &string[i + 1], count);
    file_spec->name[count] = 0;

    strncpy(dir_str, string, i);
    dir_str[i] = 0;


    if (!xvt_fsys_convert_str_to_dir(dir_str, &file_spec->dir))
    {
	printf("Unable to convert the directory into a string.");
	return (-1);
    }
    strcpy((char *) &file_spec->dir, dir_str);
    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
MUSE_API add_color_bias(GHANDLE bitmap)
#else
MUSE_API add_color_bias(bitmap)
    GHANDLE         bitmap;
#endif
{
    unsigned long   i;
    unsigned long   bitmap_size, paloff;
    BYTE pHMEM      a = (BYTE pHMEM) NULL;

    paloff = PAL_OFFSET;

    if (paloff != 0)
    {
	bitmap_size = xvt_gmem_get_size(bitmap);
	a = (BYTE pHMEM) xvt_gmem_lock(bitmap);

	for (i = 0; i < bitmap_size; i++)
	    a[i] += PAL_OFFSET;

	xvt_gmem_unlock(bitmap);
    }
    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
MUSE_API remove_color_bias(GHANDLE bitmap)
#else
MUSE_API remove_color_bias(bitmap)
    GHANDLE         bitmap;
#endif
{
    unsigned long   i;
    unsigned long   bitmap_size, paloff;
    BYTE pHMEM      a = (BYTE pHMEM) NULL;

    paloff = PAL_OFFSET;

    if (paloff != 0)
    {
	bitmap_size = xvt_gmem_get_size(bitmap);
	a = (BYTE pHMEM) xvt_gmem_lock(bitmap);

	for (i = 0; i < bitmap_size; i++)
	    a[i] -= PAL_OFFSET;

	xvt_gmem_unlock(bitmap);
    }
    return STAT_SUCCESS;
}
#endif
