/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapfeature.h>
#include <Map/mapfiles.h>
#include <Attribute/attrlist.h>

#include <limits.h>

#include <float.h>

#include <stdio.h>

#include <string.h>


/*****************************************************************************/

MapFeature::MapFeature(MapFeatureClass* fclass, int id, MapFeatureType type) {
  _fclass = fclass;
  _id = id;
  _primitive = nil;
  _type  = type;
  _attrlist = nil;
}

MapFeature::MapFeature(int id, MapFeatureType type) {
  _fclass = nil;
  _id = id;
  _primitive = nil;
  _type  = type;
  _attrlist = nil;
}

MapFeature::MapFeature() {
  _fclass = nil;
  _id = -1;
  _primitive = nil;
  _type  = UnknownType;
  _attrlist = nil;
}

MapFeature::MapFeature(MapFeature& mf) {
  _fclass = mf._fclass;
  _id = mf._id;
  _primitive = mf._primitive;
  _type  = mf._type;
  _attrlist = mf._attrlist;
  Resource::ref(_attrlist);
}

MapFeature::~MapFeature() {
}

const char* MapFeature::describe() const {
    static char buffer[BUFSIZ];
    char* typestring;
    switch (_type) {
    case PointType:
	typestring = "point";
	break;
    case LineType:
	typestring = "line";
	break;
    case AreaType:
	typestring = "area";
	break;
    case TextType:
	typestring = "text";
	break;
    case JunctionType:
	typestring = "junction";
	break;
    case RasterType:
	typestring = "raster";
	break;
    case DemType:
	typestring = "dem";
	break;
    }
    sprintf(buffer, "%s feature, id %d", typestring, _id);
    return buffer;
}

float MapFeature::unprojscale() { return 1.0; }

void MapFeature::primitive(MapPrimitive* primitive) {
    _primitive = primitive;
}

void MapFeature::edge_primitive(MapEdge* primitive) {
    if (_type==MapFeature::LineType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::face_primitive(MapFace* primitive) {
    if (_type==MapFeature::AreaType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::node_primitive(MapNode* primitive) {
    if (_type==MapFeature::JunctionType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::point_primitive(MapPoint* primitive) {
    if (_type==MapFeature::PointType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::text_primitive(MapText* primitive) {
    if (_type==MapFeature::TextType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::raster_primitive(MapRaster* primitive) {
    if (_type==MapFeature::RasterType)
	_primitive = (MapPrimitive*) primitive;
}

void MapFeature::dem_primitive(MapDem* primitive) {
    if (_type==MapFeature::DemType)
	_primitive = (MapPrimitive*) primitive;
}

MapPrimitive* MapFeature::primitive() const {
    return _primitive;
}

MapEdge* MapFeature::edge_primitive() const {
    return (MapEdge*) (_type==MapFeature::LineType ? _primitive : nil);
}

MapFace* MapFeature::face_primitive() const {
    return (MapFace*) (_type==MapFeature::AreaType ? _primitive : nil);
}

MapNode* MapFeature::node_primitive() const {
    return (MapNode*) (_type==MapFeature::JunctionType ? _primitive : nil);
}

MapPoint* MapFeature::point_primitive() const {
    return (MapPoint*) (_type==MapFeature::PointType ? _primitive : nil);
}

MapText* MapFeature::text_primitive() const {
    return (MapText*) (_type==MapFeature::TextType ? _primitive : nil);
}

MapRaster* MapFeature::raster_primitive() const {
    return (MapRaster*) (_type==MapFeature::RasterType ? _primitive : nil);
}

MapDem* MapFeature::dem_primitive() const {
    return (MapDem*) (_type==MapFeature::DemType ? _primitive : nil);
}

void* MapFeature::graphic() const {
    return _gr;
}

void MapFeature::graphic(void* g) {
    _gr = g;
}

char* MapFeature::text_attr(char* attrname) {
    return "";
}

float MapFeature::float_attr(char* attrname) {
    return FLT_MIN;
}

double MapFeature::double_attr(char* attrname) {
    return DBL_MIN;
}

short int MapFeature::short_attr(char* attrname) {
    return SHRT_MIN;
}

long int MapFeature::long_attr(char* attrname) {
    return LONG_MIN;
}

AttributeList* MapFeature::attrlist() { return _attrlist; }
void MapFeature::attrlist(AttributeList* al) { _attrlist = al; Resource::ref(_attrlist);}
