/*
 * Copyright 1993-1995 Vectaport, Jorge A. Gautier Consulting          
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

extern "C" {
#include "projects.h"
}

#include <MapUnidraw/mapdialogs.h>

#include <Unidraw/globals.h>

#include <InterViews/box.h>
#include <InterViews/button.h>
#include <InterViews/event.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/matcheditor.h>
#include <InterViews/sensor.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*****************************************************************************/

GeoPointDialog::GeoPointDialog () : BasicDialog(
    new ButtonState, "", "Enter LAT LON of point:"
) {
    _medit = new MatchEditor(state, "9999999999999999999");
    _medit->Message("");
    _medit->Match("%f %f", false);

    _units = new ButtonState('d');
    input = new Sensor;
    Ref(input);
    input->Catch(KeyEvent);

    Insert(Interior());
    SelectMessage();
}

void GeoPointDialog::Handle (Event& e) { _medit->Handle(e); }

boolean GeoPointDialog::Accept () {
    Event e;
    int v = 0;

    state->SetValue(0);
    _medit->Edit();
    state->GetValue(v);

    while (v == 0) {
	Read(e);
	Forward(e);
	state->GetValue(v);
    }

    return v == '\r';
}

void GeoPointDialog::SelectMessage () {
    _medit->Select(0, strlen(_medit->Text()));
}

Interactor* GeoPointDialog::Interior () {
    const int space = round(.5*ivcm);

    VBox* titleblock = new VBox(
        new HBox(_title, new HGlue),
        new HBox(_subtitle, new HGlue)
    );

    HBox* rbuttons = new HBox(
        new RadioButton("Decimal Degrees", _units, 'd'),
        new HGlue(space, 0),
        new RadioButton("Radians", _units, 'r')
    );
    rbuttons->Insert(new HGlue);

    return new MarginFrame(
        new VBox(
            titleblock,
            new VGlue(space),
            new Frame(new MarginFrame(_medit, 2)),
            new VGlue(space/2, 0),
            rbuttons,
            new VGlue(space),
            new HBox(
                new HGlue,
                new PushButton("Cancel", state, '\007'),
		new HGlue(space, 0),
                new PushButton("  OK  ", state, '\r')
            )
        ), space, space/2, 0
    );
}
    
boolean GeoPointDialog::GetValues (float& x, float& y) {
    char* position = nil;
    position = strdup(_medit->Text());
    boolean retval;

    if (sscanf(position,"%f %f",&x, &y) != 2) {
	x = y = 0.0;
	retval = false;
    } else {
	int unit;
	_units->GetValue(unit);

	switch (unit) {
	     case 'd':   break;
	     case 'r':   x *= RAD_TO_DEG; y *= RAD_TO_DEG; break;
	}
	retval = true;
    }
    delete position;
    return retval;
}
